package java.lang.reflect;

import sun.reflect.annotation.AnnotationSupport;
import java.util.stream.Collectors;
import java.util.LinkedHashMap;
import java.util.function.Function;
import java.util.Arrays;
import java.util.Map;
import java.util.Objects;
import sun.reflect.annotation.AnnotationType;
import java.lang.annotation.Annotation;

public interface AnnotatedElement
{
    default public boolean isAnnotationPresent(Class<? extends Annotation> clazz) {
        return this.getAnnotation(clazz) != null;
    }
    
    public abstract <T extends Annotation> T getAnnotation(Class<T> p0);
    
    public abstract Annotation[] getAnnotations();
    
    default public <T extends Annotation> T[] getAnnotationsByType(Class<T> clazz) {
        Annotation[] array = this.getDeclaredAnnotationsByType(clazz);
        if (array.length == 0 && this instanceof Class && AnnotationType.getInstance(clazz).isInherited()) {
            Class<Object> superclass = ((Class)this).getSuperclass();
            if (superclass != null) {
                array = superclass.getAnnotationsByType(clazz);
            }
        }
        return array;
    }
    
    default public <T extends Annotation> T getDeclaredAnnotation(Class<T> obj) {
        Objects.requireNonNull(obj);
        Annotation[] declaredAnnotations = this.getDeclaredAnnotations();
        int length = declaredAnnotations.length;
        int i = 0;
        while (i < length) {
            Annotation obj2 = declaredAnnotations[i];
            if (obj.equals(obj2.annotationType())) {
                return (T)obj.cast(obj2);
            }
            ++i;
        }
        return null;
    }
    
    default public <T extends Annotation> T[] getDeclaredAnnotationsByType(Class<T> clazz) {
        Objects.requireNonNull(clazz);
        return AnnotationSupport.getDirectlyAndIndirectlyPresent((Map)Arrays.stream(this.getDeclaredAnnotations()).collect(Collectors.toMap(Annotation::annotationType, Function.identity(), (annotation, p1)/* java.lang.annotation.Annotation,java.lang.annotation.Annotation, */ -> annotation, LinkedHashMap::new)), clazz);
    }
    
    public abstract Annotation[] getDeclaredAnnotations();
}
